@@ -4,22 +4,24 @@ from typing import Union, Tuple, List
 
 
 class problem:
-    def __init__(self, manifold, obj_fun, obj_grad=None, obj_hvp=None, beta:int = 0, enable_autodiff: bool = True, backbone = None, enable_jit:bool = True, autobeta_args = {},  **kwargs):
+    def __init__(self, manifold, obj_fun, obj_grad=None, obj_hvp=None, beta = 'auto', enable_autodiff: bool = True, backbone = None, enable_jit:bool = True, autobeta_args = {}, Xinit = None, **kwargs):
         
         self.manifold = manifold
         self.obj_fun = obj_fun 
 
+        self.Xinit = manifold.Init_point(Xinit = Xinit)
+        self.Xinit_vec_np = manifold.tensor2array(manifold.m2v(self.Xinit))
 
         if beta == 'auto':
             if 'max_samples' in autobeta_args.keys():
                 max_samples = autobeta_args['max_samples']
             else:
-                max_samples = 100
+                max_samples = 30
 
             if 'sample_dist' in autobeta_args.keys():
                 sample_dist = autobeta_args['sample_dist']
             else:
-                sample_dist = 0.01
+                sample_dist = 1e0
 
             if 'thresholding' in autobeta_args.keys():
                 thresholding = autobeta_args['thresholding']
@@ -29,9 +31,9 @@ class problem:
             if 'coeffs' in autobeta_args.keys():
                 coeffs = autobeta_args['coeffs']
             else:
-                coeffs = 2
+                coeffs = 2.5
 
-            X_ref = manifold.tensor2array(manifold.m2v(manifold.Init_point()))
+            X_ref = manifold.tensor2array(manifold.m2v(self.Xinit))
             fval_feas_list = []
 
             for jjj in range(max_samples):
@@ -39,9 +41,12 @@ class problem:
                 D = D/np.linalg.norm(D,2) 
                 
                 X_ref_tmp = manifold.v2m(manifold.array2tensor(  X_ref + sample_dist * D  ))
-                
-                diff_fval = obj_fun( manifold.A(X_ref_tmp) ) - obj_fun( manifold.A(manifold.A(X_ref_tmp)) )
-                diff_CX =  manifold.Feas_eval(X_ref_tmp) - manifold.Feas_eval( manifold.A(X_ref_tmp) )
+                A_X_ref_tmp = manifold.A(X_ref_tmp)
+                if manifold.manifold_type == 'P':
+                    diff_fval = obj_fun( *A_X_ref_tmp ) - obj_fun( *manifold.A(A_X_ref_tmp) )
+                else:
+                    diff_fval = obj_fun( A_X_ref_tmp ) - obj_fun( manifold.A(A_X_ref_tmp) )
+                diff_CX =  manifold.Feas_eval(X_ref_tmp) - manifold.Feas_eval( A_X_ref_tmp )
                 fval_feas_list.append( (float(diff_fval), float(diff_CX) ) )
 
             fval_processed = [max( -data_local[0]/(data_local[1]**2), thresholding ) for data_local in fval_feas_list     ]   
